#include "matlab_fcs.hh"
#include <stdexcept>
#include <cstdlib>
#include <iostream>
#include <time.h>
#include <unistd.h>

using namespace std;

namespace wpipe {

  std::string 
  datestr(unsigned long t, int len) {
    struct tm tstruct;
    time_t t0 = t;
    if (!t0) t0 = time(0);
    string s = asctime(localtime_r(&t0, &tstruct));
    return s.substr(0, len);
  }

  std::string
  deblank(const std::string& str) {
    std::string::size_type ifrst = str.find_first_not_of(" \n");
    if (ifrst == std::string::npos) return "";
    std::string::size_type ilast = str.find_last_not_of(" \n");
    if (ilast != std::string::npos) ilast++;
    return str.substr(ifrst, ilast-ifrst);
  }

  void
  error(const std::string& err) {
    throw std::runtime_error(err);
  }
  
  dble_vect
  eval(const std::string& str) {
    dble_vect r;
    const char* ptr = str.c_str();
    if (*ptr != '[') {
      r.push_back(strtod(ptr, 0));
    } else {
      ptr++;
      while (*ptr && *ptr != ']') {
	while (*ptr == ' ') ptr++;
	string xinf(ptr,3);
	char* eptr = const_cast<char*>(ptr + 3);
	if (xinf == "Inf" || xinf == "inf") r.push_back(1.0/0.0);
	else                                r.push_back(strtod(ptr, &eptr));
	ptr = eptr;
	while (*ptr == ' ') ptr++;
      }
    }
    return r;
  }
  
  str_vect
  eval_str(const std::string& str) {
    str_vect r;
    string s;
    const char* ptr = str.c_str();
    bool instr = false;
    while (*ptr && *ptr != '}') {
      switch (*ptr) {
      case '\'':
      case '"':
	if (instr) {
	  r.push_back(s);
	  s.clear();
	}
	instr = !instr;
	break;
      case '{':
      case ' ':
      case ',':
	if (!instr) break;
      default:
	if (instr) {
	  s += *ptr;
	} else {
	  error("Unexpected character");
	}
      }
      ptr++;
    }
    if (instr) error("Unterminated string");
    return r;
  }

  bool
  exist(const std::string& path, const std::string& type) {
    bool rc = false;
    if (type == "file") {
      rc = (access(path.c_str(), F_OK) == 0);
    } else {
      rc = false;
    }
    return rc;
  }

  //====================================  Greatest common divisor
  //                                      Use Euler's algorithm
  long
  gcd(long a, long b) {
    if (!a) return b;
    if (b > a) {
      long t = b;
      b = a;
      a = t;
    }
    while (b) {
      long t = b;
      b = a % b;
      a = t;
    }
    return a;
  }

  long
  nextpow2(double x) {
    double powval(1.0);
    long ipow(0);
    while (powval < x) {
      powval *= 2.0;
      ipow++;
    }
    return ipow;
  }

  double 
  polyval(const double* coef, int N, double x) {
    double sum = coef[0];
    for (int i=1; i<N; i++) {
      sum *= x;
      sum += coef[i];
    }
    return sum;
  }

  std::string 
  strrep(const std::string& str, const std::string& from, const std::string& to) {
    std::string temp(str);
    string::size_type frlen = from.size();
    if (!frlen) return temp;
    while (str.find(from) != string::npos) {
      temp.replace(str.find(from), frlen, to);
    }
    return temp;
  }

  std::string 
  tolower(const std::string& str) {
    std::string temp(str);
    string::size_type strlen = temp.size();
    if (!strlen) return temp;

    for (string::size_type i=0; i<strlen; i++) {
      temp[i] = ::tolower(temp[i]);
    }
    return temp;
  }

  std::string 
  toupper(const std::string& str) {
    std::string temp(str);
    string::size_type strlen = temp.size();
    if (!strlen) return temp;

    for (string::size_type i=0; i<strlen; i++) {
      temp[i] = ::toupper(temp[i]);
    }
    return temp;
  }


  std::string 
  unquote(const std::string& str) {
    if (str.empty()) return str;
    string::size_type front = 0;
    string::size_type last  = str.size();
    if (str[front]  == '\'' || str[front]  == '"') front++;
    if (str[last-1] == '\'' || str[last-1] == '"') last--;
    if (last <= front) return "";
    return str.substr(front, last-front);
  }

  void 
  wlog(int thresh, int level, const std::string& txt) {
    if (thresh >= level) cout << txt << endl;
  }
}
