#ifndef QUICK_SELECT_HH
#define QUICK_SELECT_HH

#include <algorithm>

template <class T>
T quick_select(T arr[], int n) 
{
  //------------------------------------
  //  Active partition limits.
  int low=0;
  int high=n-1;
  int median= (low+high)/2;

  while (high > low) {
    if (high == low + 1) {  /* Two elements only */
      if (arr[low] > arr[high]) std::swap<T>(arr[low], arr[high]);
      break;
    }

    // Find median of low, middle and high items; swap into position low
    int middle = (low + high) / 2;
    if (arr[middle] > arr[high])    std::swap<T>(arr[middle], arr[high]) ;
    if (arr[low] > arr[high])       std::swap<T>(arr[low], arr[high]) ;
    if (arr[middle] > arr[low])     std::swap<T>(arr[middle], arr[low]) ;

    // Swap low item (now in position middle) into position (low+1)
    std::swap<T>(arr[middle], arr[low+1]) ;

    // Nibble from each end towards middle, swapping items when stuck
    register T* all = arr + low;
    register T* ahh = arr + high;
    register T aLow = *all++;
    for (;;) {
      while (aLow  > *(++all)) ;
      while (*(--ahh)  > aLow) ;

      if (ahh < all) break;
      std::swap<T>(*all, *ahh) ;
    }

    // Swap middle item (in position low) back into correct position
    std::swap<T>(arr[low], *ahh) ;

    // Re-set active partition
    if (ahh <= arr + median) low = all - arr;
    if (ahh >= arr + median) high = ahh - arr - 1;
  }
  return arr[median];
}
#endif
